using System;
using System.Collections.Generic;
using System.Text;
using Cryptware.NCryptoki;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Cryptware.NDigitSign
{
    /// <summary>
    /// Allow the user to Select a certificate showing a dialog with the list of all certificates in the token.
    /// </summary>
    /// 09/12/2010
    [Guid("C8035DA2-BE08-4466-B90C-DC16521BD2F7")]
    [ClassInterface(ClassInterfaceType.AutoDual)]
    [ComVisibleAttribute(true)]
    public class SimpleCertificateSelector : ICertificateSelector
    {
        /// <summary>
        /// Selects a certificate in the specified session showing a dialog with the list of all certificates in the token.
        /// </summary>
        /// <param name="session">The cryptoki session.</param>
        /// <returns></returns>
        /// 09/12/2010
        /// 09/12/2010
        public Certificate Select(Session session)
        {
            CryptokiCollection templateCert = new CryptokiCollection();
            templateCert.Add(new ObjectAttribute(ObjectAttribute.CKA_CLASS, CryptokiObject.CKO_CERTIFICATE));
            templateCert.Add(new ObjectAttribute(ObjectAttribute.CKA_CERTIFICATE_TYPE, X509Certificate.CKC_X_509));

            CryptokiCollection objs = session.Objects.Find(templateCert, 100);

            if (objs.Count == 0)
                return null;

            SelectionDialog selDlg = new SelectionDialog(objs.GetEnumerator());
            if (selDlg.ShowDialog() == DialogResult.OK)
                return (Certificate)selDlg.SelectedItem;
            
            return null;
        }
        
    }
}
